/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.opennebula.client.host;

import java.io.ByteArrayInputStream;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import javax.xml.transform.stream.StreamSource;
import org.opennebula.client.Client;
import org.opennebula.client.OneRpcResponse;
import org.opennebula.client.host.generated.HostListType;
import org.opennebula.client.host.generated.HostShareType;
import org.opennebula.client.host.generated.HostTemplateType;
import org.opennebula.client.host.generated.HostType;

/**
 *
 * @author tsbatista
 */
public class HostOperations {


    private Client client;

    private static final Map <String,String> RPC_HOST_ACTIONS;
    static {
        Map<String, String> tempMap = new HashMap<String, String>();
        tempMap.put("ALLOCATE", "host.allocate");
        tempMap.put("DELETE", "host.delete");
        tempMap.put("ENABLE", "host.enable");
        tempMap.put("INFO", "host.info");
        tempMap.put("LIST", "hostpool.info");
        RPC_HOST_ACTIONS = Collections.unmodifiableMap(tempMap);
    }

    public HostOperations(Client client) {
        this.client = client;
    }

    public HostOperations(String username, String password, String endpoint) {
        this.client = new Client(username, password, endpoint);
    }

    public Integer allocate (String hostname, String im_mad_name,String vmm_mad_name,
            String tm_mad_name, Boolean managed) {

        OneRpcResponse resp = client.rpcCall(RPC_HOST_ACTIONS.get("ALLOCATE"), 
                hostname, im_mad_name,vmm_mad_name, tm_mad_name, managed);
        if (!resp.isSuccess()) {
            throw new RuntimeException(resp.getMessage());
        }
        return Integer.parseInt(resp.getMessage());
    }

    public void delete(Integer hostId) {
        OneRpcResponse resp = client.rpcCall(RPC_HOST_ACTIONS.get("DELETE"), hostId);
        if (!resp.isSuccess()) {
            throw new RuntimeException(resp.getMessage());
        }
    }

    public void enable(Integer hostId, Boolean enable) {
        OneRpcResponse resp = client.rpcCall(RPC_HOST_ACTIONS.get("ENABLE"), hostId, enable);
        if (!resp.isSuccess()) {
            throw new RuntimeException(resp.getMessage());
        }
    }


    public HostListType list() {
        OneRpcResponse resp = client.rpcCall(RPC_HOST_ACTIONS.get("LIST"));
        if (!resp.isSuccess()) {
            throw new RuntimeException(resp.getMessage());
        }

        ByteArrayInputStream bs = new ByteArrayInputStream(resp.getMessage().getBytes());
        HostListType hostList = null;
        try {
            JAXBContext context = JAXBContext.newInstance(HostListType.class, HostType.class, HostShareType.class, HostTemplateType.class);
            Unmarshaller u = context.createUnmarshaller();
            JAXBElement<HostListType> root = u.unmarshal(new StreamSource(bs), HostListType.class);
            hostList = root.getValue();
        } catch (JAXBException ex) {
            Logger.getLogger(HostOperations.class.getName()).log(Level.SEVERE, null, ex);
            throw new RuntimeException(ex);
        }
        return hostList;
    }

    public HostType info(int id) {
        OneRpcResponse resp = client.rpcCall(RPC_HOST_ACTIONS.get("INFO"), id);
        if (!resp.isSuccess()) {
            throw new RuntimeException(resp.getMessage());
        }

        ByteArrayInputStream bs = new ByteArrayInputStream(resp.getMessage().getBytes());
        HostType host = null;
        try {
            JAXBContext context = JAXBContext.newInstance(HostType.class, HostShareType.class, HostTemplateType.class);
            Unmarshaller u = context.createUnmarshaller();
            JAXBElement<HostType> root = u.unmarshal(new StreamSource(bs), HostType.class);
            host = root.getValue();
        } catch (JAXBException ex) {
            Logger.getLogger(HostOperations.class.getName()).log(Level.SEVERE, null, ex);
            throw new RuntimeException(ex);
        }
        return host;
    }
}
