/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.opennebula.client.vm;

import java.io.ByteArrayInputStream;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import javax.xml.transform.stream.StreamSource;
import org.opennebula.client.Client;
import org.opennebula.client.OneRpcResponse;
import org.opennebula.client.vm.generated.VmType;
import org.opennebula.client.vm.generated.VmTemplateType;
import org.opennebula.client.vm.generated.VmHistoryType;
import org.opennebula.client.vm.generated.VmListType;

/**
 *
 * @author tsbatista
 */
public class VMOperations {

    // service related stuff
    Client client;
    // rpc related stuff
    private static final Map<String, String> VM_ACTION;
    private static final Map<String, String> RPC_VM_ACTIONS;

    static {
        // initialize ACTIONS map
        Map<String, String> tempMap1 = new HashMap<String, String>();
        tempMap1.put("SHUTDOWN", "shutdown");
        tempMap1.put("HOLD", "hold");
        tempMap1.put("RELEASE", "release");
        tempMap1.put("STOP", "stop");
        tempMap1.put("CANCEL", "cancel");
        tempMap1.put("SUSPEND", "suspend");
        tempMap1.put("RESUME", "resume");
        tempMap1.put("RESTART", "restart");
        tempMap1.put("FINALIZE", "finalize");
        VM_ACTION = Collections.unmodifiableMap(tempMap1);

        Map<String, String> tempMap2 = new HashMap<String, String>();
        tempMap2.put("ALLOCATE", "vm.allocate");
        tempMap2.put("DEPLOY", "vm.deploy");
        tempMap2.put("ACTION", "vm.action");//WTF?
        tempMap2.put("MIGRATE", "vm.migrate");
        tempMap2.put("INFO", "vm.info");
        tempMap2.put("SAVEDISK", "vm.savedisk");
        tempMap2.put("LIST", "vmpool.info");
        RPC_VM_ACTIONS = Collections.unmodifiableMap(tempMap2);

    }

    public VMOperations(Client client) {
        this.client = client;
    }

    public VMOperations(String username, String Password, String endpoint) {
        this.client = new Client(username, Password, endpoint);
    }

    /**
     *
     * @param template is the template for the new virtual machine. Ath the time this is only
     *  a string, but in due time it will become a template object, when the XML is modelled
     * @return The ID of the new virtual machine
     */
    public int allocate(String template) {
        OneRpcResponse resp = client.rpcCall(RPC_VM_ACTIONS.get("ALLOCATE"), template);
        if (!resp.isSuccess()) {
            throw new RuntimeException(resp.getMessage());
        }
        return Integer.parseInt(resp.getMessage());
    }

    public void deploy(Integer vmId, Integer hostId) {
        OneRpcResponse resp = client.rpcCall(RPC_VM_ACTIONS.get("DEPLOY"), vmId, hostId);
        if (!resp.isSuccess()) {
            throw new RuntimeException(resp.getMessage());
        }
    }


    public void action(String action, Integer vmId) { // TODO: check if action is possible before calling RPC
        OneRpcResponse resp = client.rpcCall(RPC_VM_ACTIONS.get("ACTION"), VM_ACTION.get(action), vmId);
        if (!resp.isSuccess()) {
            throw new RuntimeException(resp.getMessage());
        }
    }

    public void migrate(Integer vmId, Integer hostId, Boolean liveMigration) {
        OneRpcResponse resp = client.rpcCall(RPC_VM_ACTIONS.get("MIGRATE"),vmId, hostId, liveMigration);
        if (!resp.isSuccess()) {
            throw new RuntimeException(resp.getMessage());
        }
    }

    public void saveDisk(Integer vmId, Integer diskId, Integer imageId) {
        OneRpcResponse resp = client.rpcCall(RPC_VM_ACTIONS.get("SAVEDISK"),vmId, diskId, imageId);
        if (!resp.isSuccess()) {
            throw new RuntimeException(resp.getMessage());
        }
    }

    public VmType info(int id) {
        OneRpcResponse resp = client.rpcCall(RPC_VM_ACTIONS.get("INFO"), id);
        if (!resp.isSuccess()) {
            throw new RuntimeException(resp.getMessage());
        }

        ByteArrayInputStream bs = new ByteArrayInputStream(resp.getMessage().getBytes());
        VmType vm = null;
        try {
            JAXBContext context = JAXBContext.newInstance(VmType.class, VmTemplateType.class, VmHistoryType.class);
            Unmarshaller u = context.createUnmarshaller();
            JAXBElement<VmType> root = u.unmarshal(new StreamSource(bs), VmType.class);
            vm = root.getValue();
        } catch (JAXBException ex) {
            Logger.getLogger(VMOperations.class.getName()).log(Level.SEVERE, null, ex);
            throw new RuntimeException(ex);
        }
        return vm;
    }


    /*
     * lists all vms
     *
     */
    public VmListType list(int filter, boolean detail) {
        OneRpcResponse resp = client.rpcCall(RPC_VM_ACTIONS.get("LIST"), filter, detail);
        if (!resp.isSuccess()) {
            throw new RuntimeException(resp.getMessage());
        }

        ByteArrayInputStream bs = new ByteArrayInputStream(resp.getMessage().getBytes());
        VmListType vmList = null;
        try {
            JAXBContext context = JAXBContext.newInstance(VmListType.class, VmType.class, VmTemplateType.class, VmHistoryType.class);
            Unmarshaller u = context.createUnmarshaller();
            JAXBElement<VmListType> root = u.unmarshal(new StreamSource(bs), VmListType.class);
            vmList = root.getValue();
        } catch (JAXBException ex) {
            Logger.getLogger(VMOperations.class.getName()).log(Level.SEVERE, null, ex);
            throw new RuntimeException(ex);
        }
        return vmList;
    }
}
